@@ -1,9 +1,10 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 #
 # Copyright (c) 2012-2021 Snowflake Computing Inc. All rights reserved.
 #
 
+from __future__ import annotations
+
 import binascii
 import decimal
 import time
@@ -11,7 +12,7 @@ from datetime import date, datetime
 from datetime import time as dt_t
 from datetime import timedelta
 from logging import getLogger
-from typing import Any, Callable, Dict, Optional, Tuple, Union
+from typing import Any, Callable
 
 import pytz
 
@@ -90,17 +91,17 @@ def convert_datetime_to_epoch(dt: datetime) -> float:
 
 
 def _convert_datetime_to_epoch_nanoseconds(dt: datetime) -> str:
-    return "{:f}".format(convert_datetime_to_epoch(dt)).replace(".", "") + "000"
+    return f"{convert_datetime_to_epoch(dt):f}".replace(".", "") + "000"
 
 
 def _convert_date_to_epoch_milliseconds(dt: datetime) -> str:
-    return "{:.3f}".format((dt - ZERO_EPOCH_DATE).total_seconds()).replace(".", "")
+    return f"{(dt - ZERO_EPOCH_DATE).total_seconds():.3f}".replace(".", "")
 
 
 def _convert_time_to_epoch_nanoseconds(tm: dt_t) -> str:
     return (
         str(tm.hour * 3600 + tm.minute * 60 + tm.second)
-        + "{:06d}".format(tm.microsecond)
+        + f"{tm.microsecond:06d}"
         + "000"
     )
 
@@ -134,36 +135,34 @@ def _generate_tzinfo_from_tzoffset(tzoffset_minutes: int) -> pytz._FixedOffset:
     return pytz.FixedOffset(tzoffset_minutes)
 
 
-class SnowflakeConverter(object):
+class SnowflakeConverter:
     def __init__(self, **kwargs):
-        self._parameters: Dict[str, Union[str, int, bool]] = {}
+        self._parameters: dict[str, str | int | bool] = {}
         self._use_numpy = kwargs.get("use_numpy", False) and numpy is not None
 
         logger.debug("use_numpy: %s", self._use_numpy)
 
-    def set_parameters(self, new_parameters: Dict) -> None:
+    def set_parameters(self, new_parameters: dict) -> None:
         self._parameters = new_parameters
 
     def set_parameter(self, param: Any, value: Any) -> None:
         self._parameters[param] = value
 
-    def get_parameters(self) -> Dict[str, Union[str, int, bool]]:
+    def get_parameters(self) -> dict[str, str | int | bool]:
         return self._parameters
 
-    def get_parameter(self, param: str) -> Optional[Union[str, int, bool]]:
+    def get_parameter(self, param: str) -> str | int | bool | None:
         return self._parameters.get(param)
 
-    def to_python_method(self, type_name, column) -> "SnowflakeConverterType":
+    def to_python_method(self, type_name, column) -> SnowflakeConverterType:
         """FROM Snowflake to Python Objects"""
         ctx = column.copy()
         if ctx.get("scale") is not None:
             ctx["max_fraction"] = int(10 ** ctx["scale"])
             ctx["zero_fill"] = "0" * (9 - ctx["scale"])
-        converters = ["_{type_name}_to_python".format(type_name=type_name)]
+        converters = [f"_{type_name}_to_python"]
         if self._use_numpy:
-            converters.insert(
-                0, "_{type_name}_numpy_to_python".format(type_name=type_name)
-            )
+            converters.insert(0, f"_{type_name}_numpy_to_python")
         for conv in converters:
             try:
                 return getattr(self, conv)(ctx)
@@ -304,7 +303,7 @@ class SnowflakeConverter(object):
     def _TIMESTAMP_NTZ_numpy_to_python(self, ctx):
         """TIMESTAMP NTZ to datetime64 with no timezone info is attached."""
 
-        def conv(value: str) -> "numpy.datetime64":
+        def conv(value: str) -> numpy.datetime64:
             nanoseconds = int(decimal.Decimal(value).scaleb(9))
             return numpy.datetime64(nanoseconds, "ns")
 
@@ -401,7 +400,7 @@ class SnowflakeConverter(object):
 
     def _derive_offset_timestamp(
         self, value: datetime, is_utc: bool = False
-    ) -> Tuple[float, datetime]:
+    ) -> tuple[float, datetime]:
         """Derives TZ offset and timestamp from the datetime objects."""
         tzinfo = value.tzinfo
         if tzinfo is None:
@@ -436,9 +435,7 @@ class SnowflakeConverter(object):
         (mins, secs) = divmod(r, 60)
         hours += value.days * 24
         return (
-            str(hours * 3600 + mins * 60 + secs)
-            + "{:06d}".format(value.microseconds)
-            + "000"
+            str(hours * 3600 + mins * 60 + secs) + f"{value.microseconds:06d}" + "000"
         )
 
     def to_snowflake(self, value: Any) -> Any:
@@ -453,7 +450,7 @@ class SnowflakeConverter(object):
         return int(value)
 
     def _long_to_snowflake(self, value):
-        return long(value)  # noqa: F821
+        return long(value)
 
     def _float_to_snowflake(self, value: float) -> float:
         return float(value)
@@ -582,7 +579,7 @@ class SnowflakeConverter(object):
             hour=hours, minute=mins, second=secs
         )
 
-    def _decimal_to_snowflake(self, value: decimal.Decimal) -> Optional[str]:
+    def _decimal_to_snowflake(self, value: decimal.Decimal) -> str | None:
         if isinstance(value, decimal.Decimal):
             return str(value)
 
@@ -632,9 +629,9 @@ class SnowflakeConverter(object):
                 ),
                 errno=ER_NOT_SUPPORT_DATA_TYPE,
             )
-        raise AttributeError("No method is available: {}".format(item))
+        raise AttributeError(f"No method is available: {item}")
 
-    def to_csv_bindings(self, value: Union[Tuple[str, Any], Any]) -> Union[str, None]:
+    def to_csv_bindings(self, value: tuple[str, Any] | Any) -> str | None:
         """Convert value to a string representation in CSV-escaped format to INSERT INTO."""
         if isinstance(value, tuple) and len(value) == 2:
             _type, val = value
@@ -681,7 +678,7 @@ class SnowflakeConverter(object):
             # Binary literal syntax
             return "X'{}'".format(value.decode("ascii"))
 
-        return "'{}'".format(value)
+        return f"'{value}'"
 
     @staticmethod
     def escape_for_csv(value: str) -> str:
